#ifndef COMMON_DATA_H
#define COMMON_DATA_H

#include "Config/FrendyTypes.hpp"

namespace frendy
{
  static const Integer default_data_width = 11;
  static const Integer sig_fig = 7; //number of significant figures

  static const Integer unassigned_data_val = -99999;
  static const Integer unassigned_mat_no   = -99999;
  static const Integer unassigned_mt_no    = -99999;

  static const Real8 min_value    = 1.0e-60;
  static const Real8 min_sig_dif  = 1.0e-30;
  static const Real8 min_ene_dif  = 1.0e-10;

  static const Real8 min_ene_val  = 1.0e-15;
  static const Real8 min_sig_val  = 1.0e-40;

  static const int   total_xs     = 0;
  static const int   scatter_xs   = 1;
  static const int   fission_xs   = 2;
  static const int   radiation_xs = 3;
  static const int   xs_type_no   = 4; //total, scatter, fission and radiation

  static const Real8 boltzmann_const = 8.617333262145e-5; //[eV/K]
  static const Real8 amu_n           = 1.008664904; //mass of neutron[u]
  static const Real8 amu             = 1.660538921e-24; //unified atomic mass unit[u] (NJOY:1.6605402e-24)
  static const Real8 h_bar           = 1.054571726e-27; //h/2*pi[erg s] (NJOY:1.05457266e-27)
  static const Real8 ev              = 1.602176634e-12; //electron vold[erg] (NJOY:1.60217733e-12)

  static const Real8 light_speed     = 2.99792458E+8; //[m/s]

  static const Real8 unit_mega       = 1.0E+6;
  static const Real8 unit_kilo       = 1.0E+3;
  static const Real8 unit_mili       = 1.0E-3;
  static const Real8 unit_micro      = 1.0E-6;

  static const Real8 dop_ene_max_def       = 1.0e6;  //The default value of max doppler energy
  static const Real8 integral_range_dop    = 4.0;    //Integral range for doppler broadening calculation
  static const Real8 tayler_calc_condition = 1.0e-10;
  static const Real8 min_tayler_val        = 1.0e-30;

  static const Real8 conv_th_ene           = 0.0253; //conventional thermal energy is 0.0253[eV]

  static const Integer int_const      = 1; //For identification of interpolation scheme
  static const Integer int_lin_lin    = 2;
  static const Integer int_log_lin    = 3;
  static const Integer int_lin_log    = 4;
  static const Integer int_log_log    = 5;
  static const Integer int_special_1d = 6;
}

#endif // COMMON_DATA_H
