#include "EndfUtils/GendfParser/GMFxxParserBase.hpp"

using namespace frendy;

//constructor
GMFxxParserBase::GMFxxParserBase(void)
{
  clear();

  err_obj.set_output_mode(err_obj.warning_off);
}

//destructor
GMFxxParserBase::~GMFxxParserBase(void)
{ 
  clear();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void GMFxxParserBase::store_data() 
{
  MultiParser mp_obj;

  if( mf_no == 0 )
  {
    string class_name = "GMFxxParserBase";
    string func_name  = "store_data()";

    vector<string> err_com;
    err_com.push_back("Please set MF number before storing the data.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  //Text file data check
  if( static_cast<Integer>(text.size()) < 2 )
  {
    string class_name = "GMFxxParserBase";
    string func_name  = "store_data()";

    ostringstream oss;
    oss << static_cast<Integer>(text.size());
    string str_data = "Text line number : " + oss.str();

    vector<string> err_com;
    err_com.push_back(str_data);
    err_com.push_back("The text line number is not appropriate for MF=03.");
    err_com.push_back("The text line number should be larger than 1.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  //Check MF number
  if( mp_obj.check_mf(text, mf_no) != 0)
  {
    string class_name = "GMFxxParserBase";
    string func_name  = "store_data()";

    ostringstream oss01;
    oss01 << mf_no;
    string str_data01 = "Text data at first line : " + text[0];
    string str_data02 = "File number (MF)        : " + oss01.str();

    vector<string> err_com;
    err_com.push_back(str_data01);
    err_com.push_back(str_data02);
    err_com.push_back("The MF number is different.");

    err_obj.output_runtime_error(class_name, func_name, err_com);
  }

  mp_obj.get_mat(text, mat_no);

  int     line_no = 0;
  Integer tmp_int;
  Real    tmp_real;

  //Get MT number
  mp_obj.get_mf_mt(text, tmp_int, mt_no);

  //Line No.01
  //ZA, AWR
  mp_obj.read_head(text, line_no, ZA, AWR, NL, NSIGZ, LR, NGN);

  int eg_no   = static_cast<int>(NGN) + 1; //MF=6 MT=18 has NGN+1 data
  int sigz_no = static_cast<int>(NSIGZ);
  int pl_no   = static_cast<int>(NL);

  NG2.resize(eg_no);
  IG2LO.resize(eg_no);
  DATA1.resize(eg_no);
  DATA2.resize(eg_no);
  for(int i=0; i<eg_no; i++)
  {
    NG2[i]   = -1;
    IG2LO[i] = -1;
  }

  int ng1         = 0;
  int line_no_max = static_cast<int>(text.size());
  while( line_no < line_no_max )
  {
    //Line No.02
    //TEMP, DATA1 and DATA2
    Real         temp_each;
    int          ng2_each, ig2lo_each, data_no, IG;
    vector<Real> e_vec;
    mp_obj.read_list(text, line_no, temp_each, tmp_real, ng2_each, ig2lo_each, data_no, IG, e_vec);

    if( ng1 == 0 )
    {
      TEMP = temp_each;
    }
    else
    {
      if( fabs(TEMP - temp_each) > 1.0E-3 )
      {
        string class_name = "GMFxxParserBase";
        string func_name  = "store_data()";

        ostringstream oss01, oss02, oss03, oss04, oss05;
        oss01 << mf_no;
        oss02 << mt_no;
        oss03 << IG;
        oss04 << TEMP;
        oss05 << temp_each;
        string str_data01 = "MF number           : " + oss01.str();
        string str_data02 = "MT number           : " + oss02.str();
        string str_data03 = "IG                  : " + oss03.str();
        string str_data04 = "TEMP of first grid  : " + oss04.str();
        string str_data05 = "TEMP of IG grid     : " + oss05.str();

        vector<string> err_com;
        err_com.push_back(str_data01);
        err_com.push_back(str_data02);
        err_com.push_back(str_data03);
        err_com.push_back(str_data04);
        err_com.push_back(str_data05);
        err_com.push_back("");
        err_com.push_back("Line data");
        err_com.push_back(text[line_no-1]);
        err_com.push_back("");
        err_com.push_back("TEMP data must be identical in all energy grid.");
        err_com.push_back("This GENDF file might be incorrect.");

        err_obj.output_caution(class_name, func_name, err_com);
      }
    }
    ng1++;

    int e_pos    = IG; //MF=6 MT=18 has IG=0-NGN
    NG2[e_pos]   = ng2_each;
    IG2LO[e_pos] = ig2lo_each;
    DATA1[e_pos].resize(sigz_no);
    DATA2[e_pos].resize(sigz_no);
    for(int i=0; i<sigz_no; i++)
    {
      DATA1[e_pos][i].resize(pl_no);
      DATA2[e_pos][i].resize(pl_no);
      for(int j=0; j<pl_no; j++)
      {
        DATA1[e_pos][i][j] = 0.0;
        DATA2[e_pos][i][j].resize(ng2_each - 1);
        for(int k=0; k<ng2_each - 1; k++)
        {
          DATA2[e_pos][i][j][k] = 0.0;
        }
      }
    }


    int ele_no = 0;
    for(int i=0; i<sigz_no; i++)
    {
      for(int j=0; j<pl_no; j++)
      {
        DATA1[e_pos][i][j] = e_vec[ele_no];
        ele_no++;
      }
    }

    for(int k=0; k<ng2_each - 1; k++)
    {
      for(int i=0; i<sigz_no; i++)
      {
        for(int j=0; j<pl_no; j++)
        {
          DATA2[e_pos][i][j][k] = e_vec[ele_no];
          ele_no++;
        }
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void GMFxxParserBase::clear()
{
  VectorClearer clr_vec;
  clr_vec.clear_vec_array1_str(text);

  mat_no = unassigned_mat_no;
  mf_no  = 0;
  mt_no  = unassigned_mt_no;
  ZA     = 0.0;
  AWR    = 0.0;
  NL     = 0;
  NSIGZ  = 0;
  LR     = 0;
  NGN    = 0;

  TEMP   = 0.0;
  NG2.clear();
  IG2LO.clear();

  clr_vec.clear_vec_array3_real(DATA1);
  clr_vec.clear_vec_array4_real(DATA2);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void GMFxxParserBase::set_text(vector<string> text_data)
{
  text = text_data;
  store_data();
}

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<string>  GMFxxParserBase::get_text()
{
  return text;
}

Integer         GMFxxParserBase::get_mat_no()
{
  return mat_no;
}

Integer         GMFxxParserBase::get_mf_no()
{
  return mf_no;
}

Integer         GMFxxParserBase::get_mt_no()
{
  return mt_no;
}

//Getter at line No.01
Real            GMFxxParserBase::get_ZA()
{
  return ZA;
}

Real            GMFxxParserBase::get_AWR()
{
  return AWR;
}

Integer         GMFxxParserBase::get_NL()
{
  return NL;
}

Integer         GMFxxParserBase::get_NSIGZ()
{
  return NSIGZ;
}

Integer         GMFxxParserBase::get_LR()
{
  return LR;
}

Integer         GMFxxParserBase::get_NGN()
{
  return NGN;
}

//Getter at line No.02
Real            GMFxxParserBase::get_TEMP()
{
  return TEMP;
}

vector<int>     GMFxxParserBase::get_NG2()
{
  return NG2;
}

vector<int>     GMFxxParserBase::get_IG2LO()
{
  return IG2LO;
}

vector<vector<vector<Real> > > GMFxxParserBase::get_DATA1()
{
  return DATA1;
}

vector<vector<vector<vector<Real> > > > GMFxxParserBase::get_DATA2_NG2()
{
  return DATA2;
}

vector<vector<vector<Real> > > GMFxxParserBase::get_DATA2()
{
  vector<vector<vector<Real> > > real_vec;

  int i_max = static_cast<int>(DATA2.size());
  real_vec.resize(i_max);
  for(int i=0; i<i_max; i++)
  {
    int j_max = static_cast<int>(DATA2[i].size());
    real_vec[i].resize(j_max);
    for(int j=0; j<j_max; j++)
    {
      int k_max = static_cast<int>(DATA2[i][j].size());
      real_vec[i][j].resize(k_max);
      for(int k=0; k<k_max; k++)
      {
        int l_max = static_cast<int>(DATA2[i][j][k].size());
        if( l_max == 0 )
        {
          real_vec[i][j][k] = 0.0;
        }
        else
        {
          real_vec[i][j][k] = DATA2[i][j][k][0];
        }

        if( l_max > 1 )
        {
          string class_name = "GMFxxParserBase";
          string func_name  = "vector<vector<vector<Real> > > get_DATA2()";

          ostringstream oss01, oss02, oss03, oss04, oss05, oss06, oss07, oss08, oss09;
          oss01 << mf_no;
          oss02 << mt_no;
          oss03 << i+1;
          oss04 << i_max;
          oss05 << j+1;
          oss06 << j_max;
          oss07 << k+1;
          oss08 << k_max;
          oss09 << l_max;
          string str_data01 = "MF number           : " + oss01.str();
          string str_data02 = "MT number           : " + oss02.str();
          string str_data03 = "NG                  : " + oss03.str() + " / " + oss04.str();
          string str_data04 = "NSIGZ               : " + oss05.str() + " / " + oss06.str();
          string str_data05 = "NL                  : " + oss07.str() + " / " + oss08.str();
          string str_data06 = "NG2                 : " + oss09.str();

          vector<string> err_com;
          err_com.push_back(str_data01);
          err_com.push_back(str_data02);
          err_com.push_back(str_data03);
          err_com.push_back(str_data04);
          err_com.push_back(str_data05);
          err_com.push_back(str_data06);
          err_com.push_back("NG2 is larger than 1.");
          err_com.push_back("Please use vector<vector<vector<vector<Real> > > > get_DATA2() function.");

          err_obj.output_runtime_error(class_name, func_name, err_com);
        }
      }
    }
  }

  return real_vec;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

//Setter
void GMFxxParserBase::set_mat_no(Integer int_no)
{
  mat_no = int_no;
}

void GMFxxParserBase::set_mf_no(Integer int_no)
{
  mf_no = int_no;
}

void GMFxxParserBase::set_mt_no(Integer int_no)
{
  mt_no = int_no;
}

//line No.01
void GMFxxParserBase::set_ZA(Real real_data)
{
  ZA = real_data;
}

void GMFxxParserBase::set_AWR(Real real_data)
{
  AWR = real_data;
}

void GMFxxParserBase::set_NL(Integer int_data)
{
  NL = int_data;
}
void GMFxxParserBase::set_NSIGZ(Integer int_data)
{
  NSIGZ = int_data;
}

void GMFxxParserBase::set_LR(Integer int_data)
{
  LR = int_data;
}

void GMFxxParserBase::set_NGN(Integer int_data)
{
  NGN = int_data;
}

//line No.02
void GMFxxParserBase::set_TEMP(Real real_data)
{
  TEMP = real_data;
}

void GMFxxParserBase::set_NG2(vector<int> int_data)
{
  NG2 = int_data;
}

void GMFxxParserBase::set_IG2LO(vector<int> int_data)
{
  IG2LO = int_data;
}

void GMFxxParserBase::set_DATA1(vector<vector<vector<Real> > > real_vec)
{
  DATA1 = real_vec;
}

void GMFxxParserBase::set_DATA2_NG2(vector<vector<vector<vector<Real> > > > real_vec)
{
  DATA2 = real_vec;
}

void GMFxxParserBase::set_DATA2(vector<vector<vector<Real> > > real_vec)
{
  int i_max = static_cast<int>(real_vec.size());
  DATA2.resize(i_max);
  for(int i=0; i<i_max; i++)
  {
    int j_max = static_cast<int>(real_vec[i].size());
    DATA2[i].resize(j_max);
    for(int j=0; j<j_max; j++)
    {
      int k_max = static_cast<int>(real_vec[i][j].size());
      DATA2[i][j].resize(k_max);
      for(int k=0; k<k_max; k++)
      {
        DATA2[i][j][k].resize(1);
        DATA2[i][j][k][0] = real_vec[i][j][k];
      }
    }
  }
}

