#include "ReconResonance/RmatrixLimited/endf/LIST.h"

//ClassImp( LIST );

/*! @brief Constructor */
LIST::LIST() {}

/*! @brief Constructor */
LIST::LIST( const double C1, const double C2, const int L1, const int L2, const int NPL, const int N2, const vector<double>* b ) {

  this->Set( C1,C2,L1,L2,NPL,N2, b );

}

/*! @brief Constructor */
LIST::LIST( const char* file_name ) { this->Set( file_name ); }

/*! @brief Constructor */
LIST::LIST( istream& is ) { this->Store( is ); }

/*! @brief Destructor */
LIST::~LIST() {}

/*! @brief Copy constructor */
LIST::LIST( const LIST& o ) { this->Set( o ); }

/*! @brief Assign operator */
LIST& LIST::operator=( const LIST& o ) {

  if( &o != this ) this->Set( o );  return *this;

}



double  LIST::at        ( const int i ) const { return ary[i]; }
double& LIST::at        ( const int i )       { return ary[i]; }
double  LIST::operator[]( const int i ) const { return ary[i]; }
double& LIST::operator[]( const int i )       { return ary[i]; }

int  LIST::size () const { return ary.size(); }
bool LIST::empty() const { return ary.empty(); }

LIST* LIST::resize( const int n ) {

  ary.resize(n); return this->NPL(n);

}


// ____ Store Record __________________________________________________________

void LIST::Store( istream& is ){

  is >> header;

    const int vlen = 11;  string xline;

      ary.resize( this->NPL() );

      for( int l=0; l<num_of_lines(this->NPL(),6); l++ ){

        getline( is, xline );

          for( int i=0; i<6; i++ ) {

            int j = l*6 + i;  if( j+1 > this->NPL() ) break;

            this->B(j) = dbl_endf( xline.substr( vlen*i, vlen ) );

          }

      }

  return;

}



// ____ Write Record __________________________________________________________

void LIST::Write( ostream& os, const int MAT, const int MF, const int MT, int* NS ) const {

  //if( this->NPL() == 0 ) return;

  string blank = "           ";

    os << endf( this->C1() )  << endf( this->C2() )
       << endf( this->L1() )  << endf( this->L2() )
       << endf( this->NPL() ) << endf( this->N2() )
       << endf( MAT,MF,MT,NS );

  for( int i=0; i<this->NPL(); i++  ) {

    os << endf( this->B(i) );  if( (i+1) % 6 == 0 ) os << endf( MAT,MF,MT,NS );

  }

  if( this->NPL() % 6 != 0 ) {

    for( int i=0; i<(6-this->NPL()%6); i++  ) os << blank;

    os << endf( MAT,MF,MT,NS );

  }

}


/*! @brief write record into the file */
void LIST::Write( const char* file_name ) const {

  return write( this, file_name );

}


/* __________ Set Values __________*/

LIST* LIST::Set( const LIST& o ) {

  header = o.header;  ary = o.ary;  return this;

}


LIST* LIST::Set( const double C1, const double C2, const int L1, const int L2, const int NPL, const int N2 ) {

  header.Set( C1,C2,L1,L2,NPL,N2 );  return this;

}


LIST* LIST::Set( const vector<double>* b ) {

  ary = *b;  return this->NPL( b->size() );

}


LIST* LIST::Set( const double C1, const double C2, const int L1, const int L2, const int NPL, const int N2, const vector<double>* b ) {

  return this->Set( C1,C2,L1,L2,NPL,N2 )->Set( b );

}

LIST* LIST::Set( const char* file_name ) {
// set table from the file

  return scan( this, file_name );

}




/* @brief Print tabulated data */
void LIST::Print( const int n ) const {

  //cout.unsetf();
  cout.setf( ios::scientific );
  //for( int i=0; i<npl/3; i++ ) cout << b[ i*3 ] << "  " << b[ i*3+1 ] << "  " << b[ i*3+2 ] << endl;

  for( int i=0; i<this->NPL()/n; i++ ) {

    for( int ii=0; ii<n; ii++ ) cout << this->B( i*n + ii ) << "  ";  cout << endl;

  }

  cout.unsetf( ios::scientific );

}


const double& LIST::B( const int i ) const { return ary[i]; }
double&       LIST::B( const int i )       { return ary[i]; }



void LIST::Extract( const int n, const int m, vector<double>* b ) const {

  for( int i=0; i<this->NPL()/n; i++ ) b->push_back( this->B( i*n+m ) );

  return;

}


LIST* LIST::B( const vector<double>* b ) { return this->Set(b); }


double  LIST::C1()  const { return header.C1(); }
double& LIST::C1()        { return header.C1(); }
double  LIST::C2()  const { return header.C2(); }
double& LIST::C2()        { return header.C2(); }
int     LIST::L1()  const { return header.L1(); }
int&    LIST::L1()        { return header.L1(); }
int     LIST::L2()  const { return header.L2(); }
int&    LIST::L2()        { return header.L2(); }
int     LIST::NPL() const { return header.N1(); }
int&    LIST::NPL()       { return header.N1(); }
int     LIST::N2()  const { return header.N2(); }
int&    LIST::N2()        { return header.N2(); }

LIST* LIST::C1 ( const double val ) { header.C1(val); return this; }
LIST* LIST::C2 ( const double val ) { header.C2(val); return this; }
LIST* LIST::L1 ( const int    val ) { header.L1(val); return this; }
LIST* LIST::L2 ( const int    val ) { header.L2(val); return this; }
LIST* LIST::NPL( const int    val ) { header.N1(val); return this; }
LIST* LIST::N2 ( const int    val ) { header.N2(val); return this; }

/*
LIST* LIST::PushFront( const double val ) {

  //ary.push_front( val );  this->NPL() += 1;  return this;

  this->resize( this->NPL() + 1 );

  for( int i=this->NPL()-1; i>=1; i-- ) this->B(i) = this->B(i-1); 

  this->B(0) = val;  return this;

}
*/

LIST* LIST::PushBack( const double val ) {

  ary.push_back( val );  this->NPL() += 1;  return this;

}

LIST* LIST::PushBack( const complex<double> val ) {

  ary.push_back( real(val) );
  ary.push_back( imag(val) );

  this->NPL() += 2;  return this;

}


LIST* LIST::PopFront( const int num ) {

  //for( int i=0; i<num; i++ ) ary.pop_front();  this->NPL() -= num;  return this;

  if( this->NPL() <= num ) return this->resize(0);

  for( int i=num; i<this->NPL(); i++ ) this->B(i-num) = this->B(i);

  return this->resize( this->NPL() -num );

}


LIST* LIST::PopBack( const int num ) {

  if( this->NPL() <= num ) return this->resize(0);

  for( int i=0; i<num; i++ ) ary.pop_back();  this->NPL() -= num;  return this;

}


double  LIST::Front() const { return ary.front(); }
double& LIST::Front()       { return ary.front(); }
double  LIST::Back () const { return ary.back (); }
double& LIST::Back ()       { return ary.back (); }



LIST* LIST::Scale( const double fac ) {

  for( int i=0; i<this->NPL(); i++ ) this->B(i) *= fac;  return this;

}


LIST* LIST::Clear() {

  header.Clear();  ary.clear();  return this;

}

LIST* LIST::ClearArray() {

  ary.clear();  return this->NPL(0)->N2(0);

}


bool LIST::Check() const {

  string warning = "!! LIST::Check -> found error(s)";

  if( static_cast<int>(ary.size()) != this->NPL() ) { cerr << warning << endl;  return false; }

  return true;

}



unsigned long LIST::NumberOfLines() const {

  return 1 + num_of_lines( this->NPL(), 6 );

}



bool LIST::operator==( const LIST& o ) const {

  return ( header == o.header && ary == o.ary );

}

bool LIST::operator!=( const LIST& o ) const {

  return !( *this == o );

}

bool LIST::operator< ( const LIST& o ) const {

  return ( header < o.header );

}

bool LIST::operator> ( const LIST& o ) const {

  return ( header > o.header );

}

bool LIST::operator<=( const LIST& o ) const {

  return ( header <= o.header );

}

bool LIST::operator>=( const LIST& o ) const {

  return ( header >= o.header );

}




// I/O stream

istream& operator>>( istream& is,       LIST& o ) { o.Store( is );  return is; }
ostream& operator<<( ostream& os, const LIST& o ) { o.Write( os );  return os; }


