#if !defined(___Class_LTT2)
    #define  ___Class_LTT2

#include <iostream>
#include <deque>
////#include <TObject.h>
#include "ReconResonance/RmatrixLimited/endf/Nodes.h"
#include "ReconResonance/RmatrixLimited/endf/TAB1.h"
#include "ReconResonance/RmatrixLimited/endf/Interp2.h"


class LTT2 : public virtual TAB2 {

 public:

  class Record : public TAB1 {

   public:

    Record();
    Record( const TAB1& o );

    virtual ~Record();

    double  T () const { return this->C1(); }
    double& T ()       { return this->C1(); }
    double  E () const { return this->C2(); }
    double& E ()       { return this->C2(); }
    int     LT() const { return this->L1(); }
    int&    LT()       { return this->L1(); }

    Record* T ( const double val ) { this->C1(val);  return this; }
    Record* E ( const double val ) { this->C2(val);  return this; }
    Record* LT( const int    val ) { this->L1(val);  return this; }

    ////ClassDef( LTT2::Record, 1 );

  };

 private:

  deque<LTT2::Record> array;

 public:

  LTT2();
  LTT2( const int n, const int interpcode = 2 );

  virtual ~LTT2();

  Record  at( const int i ) const;
  Record& at( const int i );

  Record  operator[]( const int i ) const;
  Record& operator[]( const int i );

  int  size () const;
  bool empty() const;

  Record  Front() const;
  Record& Front();
  Record  Back () const;
  Record& Back ();

  int  NE() const { return this->TAB2::NZ(); }
  int& NE()       { return this->TAB2::NZ(); }

  double  T ( const int i ) const { return this->at(i).T (); }
  double& T ( const int i )       { return this->at(i).T (); }
  double  E ( const int i ) const { return this->at(i).E (); }
  double& E ( const int i )       { return this->at(i).E (); }
  int     LT( const int i ) const { return this->at(i).LT(); }
  int&    LT( const int i )       { return this->at(i).LT(); }

  void Store( istream& is );
  void Read ( const char* file_name );
  void Write( ostream& os = cout, const int MAT = 0, const int MF = 0, const int MT = 0, int* NS = NULL ) const;
  void Write( const char* file_name ) const;

  unsigned long NumberOfLines() const;

  LTT2* Set( const LTT2& o );
  LTT2* Set( const char* file_name );
  LTT2* Set( const int n, const int interpcode = 2 );

  LTT2* Clear();

  LTT2* PushFront( const Record& o, const int interpcode = 2 );
  LTT2* PushBack ( const Record& o, const int interpcode = 2 );
  LTT2* PushFront( const LTT2&   o, const int interpcode = 2 );
  LTT2* PushBack ( const LTT2&   o, const int interpcode = 2 );

  LTT2* PutClone( const double elab );

  LTT2* PopFront( const int num = 1 );
  LTT2* PopBack ( const int num = 1 );

  LTT2* Erase( const double E );

  LTT2* Thin();
  LTT2* Thin( const double eps );

  LTT2* CutBelow( const double elab, const double eps = 1.e-3 );
  LTT2* CutAbove( const double elab, const double eps = 1.e-3 );

  bool Find( const double E ) const;
  bool Seek( const double E ) const;
  bool Seek( const double E, int* index, int* interpcode ) const;

  LTT2* ScaleEnergy( const double fac );

  void   Interp( Record* result,    const double eps = 1.e-3 ) const;
  void   Interp( TAB1*   result,    const double eps = 1.e-3 ) const;
  Record Interp( const double elab, const double eps = 1.e-3 ) const;

  LTT2* Weight( const double power = 1.0 );

  LTT2* Average( const int ndiv );

  double f( const double elab, const double cosangle ) const;

  void   get_f_da( TAB1* result, const double eps = 1.e-3 ) const;


  double I() const;

  LTT2& operator+=( const double val );
  LTT2& operator-=( const double val );
  LTT2& operator*=( const double val );
  LTT2& operator/=( const double val );

  //void   Integral( TAB1* ans, const int nrandom ) const;
  //void   Integral( TAB1* ans, const int nrandom ) const;

  ////ClassDef( LTT2, 1 );

};

#endif


