#if !defined(___Class_TAB1)
    #define  ___Class_TAB1

#include <vector>
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <string>
#include <algorithm>
#include <iomanip>

#include "ReconResonance/RmatrixLimited/numeric/Datum.h"
#include "ReconResonance/RmatrixLimited/endf/TAB2.h"
#include "ReconResonance/RmatrixLimited/endf/LIST.h"
#include "ReconResonance/RmatrixLimited/endf/TOF.h"
#include "ReconResonance/RmatrixLimited/endf/Numeric.h"
#include "ReconResonance/RmatrixLimited/endf/Interp1.h"
//using namespace std;


class TAB1 {

   TAB2 header;  deque<Datum> ary;

  public:

    TAB1();
    TAB1( istream& is );
    TAB1( const char* file_name );
    TAB1( const char* file_name, const int interpcode );
    TAB1( const char* file_name, const int index_x, const int index_y, const int interpcode );
    TAB1( const int np, const int interpcode = 2 );
    TAB1( const double xmin, const double xmax, const int ndiv = 1, const int interpcode = 2 );
    TAB1( const vector<Numeric::Datum>* o, const int interpcode = 2 );
    TAB1( const double* x,                  const int np,   const int interpcode = 2 );
    TAB1( const double* x, const double* y, const int np,   const int interpcode = 2 );
    TAB1( const vector<double>* x,                          const int interpcode = 2 );
    TAB1( const vector<double>* x, const vector<double>* y, const int interpcode = 2 );
    TAB1( const deque <double>* x,                          const int interpcode = 2 );
    TAB1( const deque <double>* x, const deque <double>* y, const int interpcode = 2 );

    virtual ~TAB1();

    TAB1           ( const TAB1& o );
    TAB1& operator=( const TAB1& o );

    Datum  at        ( const int i ) const;
    Datum& at        ( const int i );
    Datum  operator[]( const int i ) const;
    Datum& operator[]( const int i );

    Datum  front() const;
    Datum  back () const;
    Datum& front();
    Datum& back ();

    int  size () const;
    bool empty() const;


    TAB1* Store( istream& is );
    int   Read( const char* file_name );
    void  Write( ostream& os = cout, const int MAT = 0, const int MF = 0, const int MT = 0, int* NS = NULL ) const;
    void  Write( const char* file_name ) const;

    TAB1* Set( const TAB1& o, const bool option = false );
    TAB1* Set( const int np, const int interpcode = 2 );
    TAB1* Set( const double xmin, const double xmax, const int ndiv = 1, const int interpcode = 2 );
    TAB1* Set( const vector<Numeric::Datum>* a,                          const int interpcode = 2 );
    TAB1* Set( const double* x,                  const int np,   const int interpcode = 2 );
    TAB1* Set( const double* x, const double* y, const int np,   const int interpcode = 2 );
    TAB1* Set( const vector<double>* x,                          const int interpcode = 2 );
    TAB1* Set( const vector<double>* x, const vector<double>* y, const int interpcode = 2 );
    TAB1* Set( const deque<double>*  x,                          const int interpcode = 2 );
    TAB1* Set( const deque<double>*  x, const deque<double>*  y, const int interpcode = 2 );
    TAB1* Set( const char* file_name );
    TAB1* Set( const char* file_name, const int interpcode );
    TAB1* Set( const char* file_name, const int index_x, const int index_y, const int interpcode );

    double  X( const int i ) const;
    double  Y( const int i ) const;
    double& X( const int i );
    double& Y( const int i );

    double Ymin() const;
    double Ymax() const;

    Datum  Front() const;
    Datum  Back () const;
    Datum& Front();
    Datum& Back ();

    TAB1* Front( const Datum& o );
    TAB1* Back ( const Datum& o );

    bool CheckDuplicated( const double x ) const;

    void Diagnose( Document* result, int* ns ) const;

    TAB1* Narrow ( const double x1, const double x2 );
    TAB1  Extract( const double x1, const double x2 ) const;

    TAB1* CutAbove( const double x );
    TAB1* CutBelow( const double x );
    TAB1* PutClone( const double x );
    TAB1* PutZero ( const double x );

    TAB1* PushFront( const double x, const double y = 0.0, const int interpcode = 2, const bool inspect = true );
    TAB1* PushBack ( const double x, const double y = 0.0, const int interpcode = 2, const bool inspect = true );

    TAB1* PushFront( const Datum& o, const int interpcode = 2, const bool inspect = true );
    TAB1* PushBack ( const Datum& o, const int interpcode = 2, const bool inspect = true );

    TAB1* PopFront( const int num = 1 );
    TAB1* PopBack ( const int num = 1 );

    TAB1* PushFront( const TAB1& o, const int interpcode = 2, const bool inspect = true );
    TAB1* PushBack ( const TAB1& o, const int interpcode = 2, const bool inspect = true );

    TAB1* ResizeFront( const TAB1& o );
    TAB1* ResizeBack ( const TAB1& o );

    TAB1* Replace( const TAB1& o );

    TAB1* Update( const bool option = true );
    TAB1* Clear();
    TAB1* ClearArray();

    TAB1* Put    ( const double x );
    TAB1* PutSame( const double x );

    bool  CheckDuplication( int* index ) const;

    TAB1* Remove( const int index, const int interpcode = 0 );
    TAB1* Remove( const double  x, const int interpcode = 0 );

    TAB1* Truncate( const double y_min = 1.e-16 );
    TAB1* Trim();
    TAB1* Correct();

    TAB1* Unique();

    TAB1* TrimLower( const double x );
    TAB1* TrimUpper( const double x );

    bool Find( const double& x, const double eps ) const;
    bool Find( const double x ) const;
    bool Find( const double x, int* index ) const;

    double I( const int i ) const;
    double I() const;
    double I( const double x1, const double x2 ) const;

    TAB1* SetY( const double val );

    TAB1* ScaleX( const double fact );
    TAB1* ScaleY( const double fact );

    TAB1* ScaleX( double (*func)( const double& ) );
    TAB1* ScaleY( double (*func)( const double& ) );

    TAB1* ShiftX( const double val );
    TAB1* ShiftY( const double val );

    TAB1* ShiftX( double (*func)( const double& ) );
    TAB1* ShiftY( double (*func)( const double& ) );

    TAB1* PowerX( const double val );
    TAB1* PowerY( const double val );

    TAB1* Cos2a();

    double Interp    ( const double x ) const;
    double operator()( const double x ) const;

    double Average( const double x1, const double x2 ) const;

    TAB1*  Average( const double dx );
    TAB1*  Average( const double* x, const int nbins );
    TAB1*  Average( const double* x, const int nbins, const double xmin );
    TAB1*  Average( const int ndiv );

    void PrintColumns( ostream& os = cout ) const;
    void PrintColumns( const char*  file_name ) const;
    void PrintColumns( const string file_name ) const;

    TAB1* C1( const double val );
    TAB1* C2( const double val );
    TAB1* L1( const int    val );
    TAB1* L2( const int    val );
    TAB1* NR( const int    val );
    TAB1* NP( const int    val );

    double  C1() const;
    double& C1();
    double  C2() const;
    double& C2();
    int     L1() const;
    int&    L1();
    int     L2() const;
    int&    L2();
    int     NR() const;
    int&    NR();
    int     NP() const;
    int&    NP();

    int  NBT( const int i ) const;
    int& NBT( const int i );
    int  INT( const int i ) const;
    int& INT( const int i );

    int InterpCode( const int index ) const;

    //TAB1( const TAB1& tab1 );
    //TAB1& operator= ( const TAB1& tab1 );
    TAB1& operator<<( const TAB1& o );
    TAB1& operator+=( const TAB1& o );
    TAB1& operator-=( const TAB1& o );
    TAB1& operator*=( const TAB1& o );
    TAB1& operator/=( const TAB1& o );
    TAB1& operator+=( const double val );
    TAB1& operator-=( const double val );
    TAB1& operator*=( const double val );
    TAB1& operator/=( const double val );

    TAB1  operator+( const TAB1& o );
    TAB1  operator-( const TAB1& o );
    TAB1  operator*( const TAB1& o );
    TAB1  operator/( const TAB1& o );
    TAB1  operator+( const double val );
    TAB1  operator-( const double val );
    TAB1  operator*( const double val );
    TAB1  operator/( const double val );

    bool operator==( const TAB1& o ) const;
    bool operator!=( const TAB1& o ) const;
    bool operator< ( const TAB1& o ) const;
    bool operator> ( const TAB1& o ) const;
    bool operator<=( const TAB1& o ) const;
    bool operator>=( const TAB1& o ) const;

    TAB1* Add     ( const TAB1& o, const double eps = 1.e-3 );
    TAB1* Subtract( const TAB1& o, const double eps = 1.e-3 );
    TAB1* Multiply( const TAB1& o, const double eps = 1.e-3 );
    TAB1* Divide  ( const TAB1& o, const double eps = 1.e-3 );

    TAB1* Weight( const double power = 1.0 );

    TAB1* Normalize( const TAB1& o );

    TAB1 Difference( const TAB1& o, const bool is_percent = false ) const;

    bool Check() const;
    bool Seek( const double x ) const;
    bool Seek( const double x, int* index, int* interpcode ) const;

    unsigned long NumberOfLines() const;

    TAB1* Reverse();

    bool Empty() const { return this->NP() == 0 ? true : false; }

    TAB1* Linearize  (                       const double eps = 1.e-3 );
    TAB1* Reconstruct( const int interpcode, const double eps = 1.e-3 );

    void Break( vector<TAB1>* result ) const;

    void Split( vector<TAB1>* result ) const;

    //ClassDef( TAB1, 1 );

};

void Uniform ( TAB1* a, TAB1* b, const double eps =1.e-3 );

void add_logarithmic_points( vector<double>* o, const int ndiv = 13 );
void add_logarithmic_points( deque<double>* o, const int ndiv = 13 );

void reconstruct( TAB1* o, const int& interpcode, const double& eps );

void thin( TAB1* a, const double eps );

int search( const double& x, const TAB1* o );


istream& operator>>( istream& is,       TAB1& o );
ostream& operator<<( ostream& os, const TAB1& o );

#endif

